from contextlib import contextmanager

from infi.shell import runtime
from infi.shell.config import config, altered_config

from ...utils import input_password
from ..system.user import Roles

PRIVILEGED_ROLES = set((Roles.INFINIDAT, Roles.ADMIN))

def is_privileged_user(user):
    """ Check whether or not a user entity is a root or admin user """
    return any(r.upper() in PRIVILEGED_ROLES for r in user.roles)

def user_has_role(user, role):
    """ Check whether or not a user entity has the specified role """
    return role.upper() in (r.upper() for r in user.roles)

def is_pool_admin(user):
    """ Check whether or not a user entity is a pool admin """
    return user_has_role(user, Roles.POOLADMIN)

@contextmanager
def increased_api_timeout(timeout_seconds=300):
    """ Temporarily increase API timeout to 5 minutes, or to the specified
    value. If the current API timeout is greater than 5 minutes or the specified
    value, nothing is changed. """
    if (config.api_request_timeout_seconds or 0) > timeout_seconds:
        yield
    else:
        with altered_config(api_request_timeout_seconds=timeout_seconds):
            yield

def are_none(*values):
    return all(v is None for v in values)

def any_is_none(*values):
    return any(v is None for v in values)

def is_detailed_query():
    return runtime.options.get("detailed", False)

def input_password_check_credentials(prompt=None, username=None, interaction_error=None):
    """ Input a user's password, and validate the credentials. When ``username``
    is ``None``, use the current logged-in user to validate the credentials. """
    from .system import system
    return system.check_credentials(username or system.user.username, input_password(prompt, interaction_error=interaction_error))
