from infi.shell.exceptions import CommandFailed

class Prefixless(object):
    ''' Mixin for exceptions of which formatted representation doesn't include
    the exception name, only a message. '''
    pass

class APIFailure(CommandFailed):
    pass

class UnexpectedAPIResponse(APIFailure):
    pass

class ConnectionFailed(APIFailure):
    pass

class ManagementException(APIFailure):
    def __init__(self, code, message, http_code):
        self.code = code
        self.message = message
        self.http_code = http_code
        super(ManagementException, self).__init__(self.message)

class Unauthorized(APIFailure):
    def __init__(self, *_):
        super(Unauthorized, self).__init__("Bad credentials")

class NotFound(APIFailure):
    pass

class ConfigurationConflict(APIFailure):
    pass

class InternalServerError(APIFailure):
    pass

class NotSupported(APIFailure):
    pass

class ServiceTemporarilyUnavailable(APIFailure):
    pass

class Forbidden(APIFailure):
    pass

class UnexpectedServerResponse(APIFailure):
    def __init__(self, http_code, msg=None):
        if msg:
            msg = "Server returned an unexpected HTTP code ({}) with the following error: {}".format(http_code, msg)
        else:
            msg = "Server returned an unexpected HTTP code: {}".format(http_code)
        super(UnexpectedServerResponse, self).__init__(msg)

class BadPageStructure(CommandFailed):
    pass

class NoSuchEntity(APIFailure, Prefixless):
    pass

class InvalidConstant(CommandFailed, Prefixless):
    pass

class InvalidSystemState(CommandFailed):
    pass